package com.cweb.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.cweb.config.SysConfig;
import com.cweb.config.SysConst;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.security.SessionObject;
import com.hai.common.security.UserCenter;
import com.hai.common.utils.CoordCommonUtil;
import com.hai.common.utils.ResponseMsgUtil;
import com.hai.config.CommonSysConst;
import com.hai.config.QianZhuConfig;
import com.hai.entity.*;
import com.hai.enum_type.ProductImgEnum;
import com.hai.model.ApiStarbucksProductsModel;
import com.hai.model.BsProductConfigModel;
import com.hai.model.HighUserModel;
import com.hai.model.ResponseData;
import com.hai.service.ApiProductService;
import com.hai.service.BsConfigService;
import com.hai.service.HighOrderService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.w3c.dom.ls.LSException;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;


@Controller
@RequestMapping(value = "/highThirdParty")
@Api(value = "订单接口")
public class HighThirdPartyController {

    private static Logger log = LoggerFactory.getLogger(HighThirdPartyController.class);

    @Resource
    private ApiProductService apiProductService;

    @Resource
    private BsConfigService bsConfigService;

    @Resource
    private HighOrderService highOrderService;

    @Autowired
    private UserCenter userCenter;


    @RequestMapping(value = "/getAllCity", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询所有城市")
    public ResponseData getAllCity(@RequestParam(name = "cityName", required = false) String cityName) {
        try {
            return ResponseMsgUtil.success(apiProductService.getAllCity(cityName));

        } catch (Exception e) {
            log.error("HighUserCardController --> oilCardRefund() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/getStarbucksProducts", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询星巴克商品")
    public ResponseData getStarbucksProducts(@RequestParam(name = "name", required = false) String name) {
        try {

            List<String> CategoryNameList = apiProductService.getCategoryName();

            List<ApiStarbucksProducts> starbucksProductsList = new ArrayList<>();
            int i = 9999;
            for (String s : CategoryNameList) {
                i++;
                ApiStarbucksProducts starbucksProducts = new ApiStarbucksProducts();
                Map<String, Object> map = new HashMap<>();
                map.put("name" , name);
                map.put("categoryName" , s);
                List<ApiStarbucksProducts> list = apiProductService.getStarbucksProducts(map);
                for (ApiStarbucksProducts apiStarbucksProducts : list) {
                    apiStarbucksProducts.setExt1(String.valueOf(i));
                    starbucksProductsList.add(apiStarbucksProducts);
                }
                starbucksProducts.setName(s);
                starbucksProducts.setExt2(SysConst.getSysConfig().getImgUrl() + ProductImgEnum.getNameByImgUrl(s));
                starbucksProducts.setId((long) i);
                starbucksProductsList.add(starbucksProducts);
            }

            return ResponseMsgUtil.success(starbucksProductsList);

        } catch (Exception e) {
            log.error("HighUserCardController --> oilCardRefund() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/getStarbucksStore", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询星巴克门店")
    public ResponseData getStarbucksStore(@RequestParam(name = "lat", required = true) String lat,
                                          @RequestParam(name = "lon", required = true) String lon) {
        try {
            JSONObject jsonObject = QianZhuConfig.getStarbucksStore(lat , lon);
            if (jsonObject.getBoolean("success")) {
                return ResponseMsgUtil.success(jsonObject.getJSONArray("data"));
            } else {
                log.error("getRechargeProduct error!", "查询失败！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR , jsonObject.getString("message"));
            }

        } catch (Exception e) {
            log.error("HighUserCardController --> oilCardRefund() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/findStarbucksProducts", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "根据id查询详情产品详情")
    public ResponseData findStarbucksProducts(@RequestParam(value = "id", required = true) Long id) {
        try {

            BsProductConfig bsProductConfig = apiProductService.findProductConfigByGoodsId(id);

            if (bsProductConfig == null) {
                log.error("findStarbucksProducts error!", "查询失败！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR , "未查询到当前商品");
            }

            ApiStarbucksProducts apiStarbucksProducts = apiProductService.findStarbucksProducts(bsProductConfig.getProductId());
            apiStarbucksProducts.setSalesVentiPrice(apiStarbucksProducts.getMarketVentiPrice().multiply(bsProductConfig.getDiscount()).divide(new BigDecimal(100)));
            apiStarbucksProducts.setSalesTallPrice(apiStarbucksProducts.getMarketTallPrice().multiply(bsProductConfig.getDiscount()).divide(new BigDecimal(100)));
            apiStarbucksProducts.setSalesGrandePrice(apiStarbucksProducts.getMarketGrandePrice().multiply(bsProductConfig.getDiscount()).divide(new BigDecimal(100)));

            ApiStarbucksProductsModel starbucksProductsModel = new ApiStarbucksProductsModel();
            BeanUtils.copyProperties(apiStarbucksProducts, starbucksProductsModel);

            Map<String, Object> listMap = new HashMap<>();
            listMap.put("productType", "5");
            listMap.put("returnType", 1);
            listMap.put("sourceId", bsProductConfig.getId());
            listMap.put("platformId", null);

            starbucksProductsModel.setIntegralDiscount(bsConfigService.getProductDiscountByMap(listMap).getDiscount());
            starbucksProductsModel.setProductPayTypeString(bsConfigService.getProductPayTypeByMap(listMap));
            starbucksProductsModel.setProductPlatformString(bsConfigService.getProductPlatformByMap(listMap));

            return ResponseMsgUtil.success(starbucksProductsModel);


        } catch (Exception e) {
            log.error("OutRechargePriceController --> findById() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/addOrder", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "增加订单")
    public ResponseData addOrder(@RequestBody JSONObject object, HttpServletRequest request) {
        try {

            // 用户
            SessionObject sessionObject = userCenter.getSessionObject(request);
            HighUserModel userInfoModel = (HighUserModel) sessionObject.getObject();


            if (StringUtils.isBlank(object.getString("customerMobile")) ||
                    StringUtils.isBlank(object.getString("regionId")) ||
                    StringUtils.isBlank(object.getString("productType")) ||
                    object.getJSONArray("orderItems") == null
            ) {
                log.error("addOrder error!");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }

            object.put("userId", userInfoModel.getHighUser().getId());

            highOrderService.insertThirdProduct(object);

            return ResponseMsgUtil.success("下单成功");

        } catch (Exception e) {
            log.error("HighOrderController -> addOrder() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/getKfcStoresList", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询肯德基门店")
    public ResponseData getKfcStoresList(@RequestParam(name = "storeName", required = false) String storeName,
                                         @RequestParam(name = "cityCode", required = true) String cityCode,
                                         @RequestParam(name = "lat", required = true) String lat,
                                          @RequestParam(name = "lon", required = true) String lon) {
        try {

            Map<String, Object> map = new HashMap<>();
            map.put("storeName" , storeName);
            map.put("cityCode" , cityCode);

            List<ApiKfcStores> list = apiProductService.getKfcStoresList(map);

            if (StringUtils.isNotBlank(lat)){
                for (ApiKfcStores stores : list) {
                    double distance = CoordCommonUtil.getDistance(Double.valueOf(stores.getLat()), Double.valueOf(stores.getLon()), Double.valueOf(lat), Double.valueOf(lon));
                    stores.setDistance(String.valueOf(Math.round(distance/100d)/10d));
                }
            }

            return ResponseMsgUtil.success(list.stream().sorted(Comparator.comparing(ApiKfcStores::getDistance)).collect(Collectors.toList()));

        } catch (Exception e) {
            log.error("HighUserCardController --> oilCardRefund() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/listByStoreCode", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询肯德基门店菜单")
    public ResponseData listByStoreCode(@RequestParam(name = "storeCode", required = false) String storeCode) {
        try {

            JSONObject jsonObject = QianZhuConfig.listByStoreCode(storeCode);

            JSONArray array = jsonObject.getJSONArray("data");

            Map<String, Object> listMap = new HashMap<>();
            listMap.put("productType", "2");
            listMap.put("companyId", 2);

            List<BsProductConfig> productConfigs = apiProductService.getProductConfig(listMap);

            for (Object data : array) {
                JSONObject dataObject = (JSONObject) data;
                if (dataObject.getJSONArray("products") != null) {
                    for (Object products : dataObject.getJSONArray("products")) {
                        JSONObject productsObject = (JSONObject) products;
                        BigDecimal productsPrice = productsObject.getBigDecimal("price");
                        productsObject.put("salePrice" , productsPrice.add(productsPrice.multiply(productConfigs.get(0).getDiscount()).divide(new BigDecimal(100))));

                        if (productsObject.getJSONArray("specGroups") != null) {
                            for (Object specGroups : productsObject.getJSONArray("specGroups")) {
                                JSONObject specGroupsObject = (JSONObject) specGroups;
                                BigDecimal specGroupsPrice = specGroupsObject.getBigDecimal("price");
                                specGroupsObject.put("salePrice" , specGroupsPrice.add(specGroupsPrice.multiply(productConfigs.get(0).getDiscount()).divide(new BigDecimal(100))));
                            }
                        }
                    }
                }
            }

            if (jsonObject.getBoolean("success")) {
                return ResponseMsgUtil.success(array);
            } else {
                log.error("getRechargeProduct error!", "查询失败！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR , jsonObject.getString("message"));
            }

        } catch (Exception e) {
            log.error("HighUserCardController --> oilCardRefund() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }
}
